@@ -2,7 +2,7 @@
 /*
  * libopenraw - cr2file.cpp
  *
- * Copyright (C) 2006-2020 Hubert Figuière
+ * Copyright (C) 2006-2022 Hubert Figuière
  * Copyright (C) 2008 Novell, Inc.
  *
  * This library is free software: you can redistribute it and/or
@@ -23,6 +23,7 @@
 #include <stddef.h>
 #include <cstdint>
 #include <vector>
+#include <limits>
 #include <memory>
 
 #include <libopenraw/cameraids.h>
@@ -49,6 +50,10 @@ namespace OpenRaw {
 
 namespace Internals {
 
+/** @addtogroup canon
+ * @{
+ */
+
 #define OR_MAKE_CANON_TYPEID(camid)                                            \
     OR_MAKE_FILE_TYPEID(OR_TYPEID_VENDOR_CANON, camid)
 
@@ -253,11 +258,14 @@ static const BuiltinColourMatrix s_matrices[] = {
       0,
       0,
       { 6602, -841, -939, -4472, 12458, 2247, -975, 2039, 6148 } },
+    { OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M2),
+      0,
+      0,
+      { 6400, -480, -888, -5294, 13416, 2047, -1296, 2203, 6137 } },
     { OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M3),
       0,
       0,
       { 6362, -823, -847, -4426, 12109, 2616, -743, 1857, 5635 } },
-    // From DNG Converter 10.3
     { OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M5),
       0,
       0,
@@ -456,6 +464,7 @@ const IfdFile::camera_ids_t Cr2File::s_def[] = {
     { "Canon EOS M", OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M) },
     { "Canon EOS M10", OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M10) },
     { "Canon EOS M100", OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M100) },
+    { "Canon EOS M2", OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M2) },
     { "Canon EOS M3", OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M3) },
     { "Canon EOS M5", OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M5) },
     { "Canon EOS M6", OR_MAKE_CANON_TYPEID(OR_TYPEID_CANON_EOS_M6) },
@@ -640,8 +649,9 @@ void Cr2File::getRawBytes(RawData &data, uint32_t offset, uint32_t byte_length,
       RawDataPtr dData = decomp.decompress();
       if (dData) {
         LOGDBG1("Out size is %dx%d\n", dData->width(), dData->height());
-        // must re-set the cfaPattern
+        // must re-set the cfaPattern and photometric interpretation
         dData->setCfaPatternType(data.mosaicInfo()->patternType());
+        dData->setPhotometricInterpretation(data.getPhotometricInterpretation());
         data.swap(*dData);
         if (!isCr2()) {
             // In TIF the raw image width needs to be divided by two
@@ -749,6 +759,8 @@ bool Cr2File::isCr2()
             && (OR_GET_FILE_TYPEID_CAMERA(typeId()) != OR_TYPEID_CANON_1DS));
 }
 
+/** @} */
+
 }
 }
 
