@@ -1,7 +1,7 @@
 /*
  * libopenraw - ifdentry.hpp
  *
- * Copyright (C) 2006-2017 Hubert Figuière
+ * Copyright (C) 2006-2020 Hubert Figuière
  *
  * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -18,9 +18,7 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-
-#ifndef OR_INTERNALS_IFDENTRY_H
-#define OR_INTERNALS_IFDENTRY_H
+#pragma once
 
 #include <stddef.h>
 #include <stdint.h>
@@ -44,11 +42,14 @@ class MetaValue;
 
 namespace Internals {
 
-class IfdFileContainer;
+/** @addtogroup ifd_parsing
+ * @{
+ */
+class IfdDir;
 
 class IfdEntry;
 
-/** Describe and IFDType */
+/** @brief Describe an IFD type */
 template <typename T>
 struct IfdTypeTrait
 {
@@ -56,8 +57,6 @@ struct IfdTypeTrait
 	static const size_t   size; /**< the storage size unit in IFD*/
 	static T EL(const uint8_t* d, size_t len) noexcept;
 	static T BE(const uint8_t* d, size_t len) noexcept;
-	static T get(IfdEntry & e, uint32_t idx = 0, bool ignore_type = false)
-		noexcept(false);
 };
 
 
@@ -73,6 +72,17 @@ inline uint8_t IfdTypeTrait<uint8_t>::BE(const uint8_t* b, size_t) noexcept
 	return *b;
 }
 
+template <>
+inline int8_t IfdTypeTrait<int8_t>::EL(const uint8_t* b, size_t) noexcept
+{
+	return *b;
+}
+
+template <>
+inline int8_t IfdTypeTrait<int8_t>::BE(const uint8_t* b, size_t) noexcept
+{
+	return *b;
+}
 
 template <>
 inline uint16_t IfdTypeTrait<uint16_t>::EL(const uint8_t* b, size_t) noexcept
@@ -86,6 +96,20 @@ inline uint16_t IfdTypeTrait<uint16_t>::BE(const uint8_t* b, size_t) noexcept
 	return BE16(b);
 }
 
+template <>
+inline int16_t IfdTypeTrait<int16_t>::EL(const uint8_t* b, size_t) noexcept
+{
+	uint16_t uns = EL16(b);
+	return *(int16_t*)&uns;
+}
+
+template <>
+inline int16_t IfdTypeTrait<int16_t>::BE(const uint8_t* b, size_t) noexcept
+{
+	uint16_t uns = BE16(b);
+	return *(int16_t*)&uns;
+}
+
 template <>
 inline uint32_t IfdTypeTrait<uint32_t>::EL(const uint8_t* b, size_t) noexcept
 {
@@ -98,6 +122,18 @@ inline uint32_t IfdTypeTrait<uint32_t>::BE(const uint8_t* b, size_t) noexcept
 	return BE32(b);
 }
 
+template <>
+inline int32_t IfdTypeTrait<int32_t>::EL(const uint8_t* b, size_t) noexcept
+{
+	return EL32(b);
+}
+
+template <>
+inline int32_t IfdTypeTrait<int32_t>::BE(const uint8_t* b, size_t) noexcept
+{
+	return BE32(b);
+}
+
 template <>
 inline std::string IfdTypeTrait<std::string>::EL(const uint8_t* b, size_t len) noexcept
 {
@@ -123,65 +159,75 @@ inline std::string IfdTypeTrait<std::string>::BE(const uint8_t* b, size_t len) n
 }
 
 template <>
-inline IFD::Rational IfdTypeTrait<IFD::Rational>::EL(const uint8_t* b, size_t) noexcept
+inline IFD::ORRational IfdTypeTrait<IFD::ORRational>::EL(const uint8_t* b, size_t) noexcept
 {
-    IFD::Rational r;
+    IFD::ORRational r;
     r.num = EL32(b);
     r.denom = EL32(b + 4);
     return r;
 }
 
 template <>
-inline IFD::Rational IfdTypeTrait<IFD::Rational>::BE(const uint8_t* b, size_t) noexcept
+inline IFD::ORRational IfdTypeTrait<IFD::ORRational>::BE(const uint8_t* b, size_t) noexcept
 {
-    IFD::Rational r;
+    IFD::ORRational r;
     r.num = BE32(b);
     r.denom = BE32(b + 4);
     return r;
 }
 
 template <>
-inline IFD::SRational IfdTypeTrait<IFD::SRational>::EL(const uint8_t* b, size_t) noexcept
+inline IFD::ORSRational IfdTypeTrait<IFD::ORSRational>::EL(const uint8_t* b, size_t) noexcept
 {
-    IFD::SRational r;
+    IFD::ORSRational r;
     r.num = EL32(b);
     r.denom = EL32(b + 4);
     return r;
 }
 
 template <>
-inline IFD::SRational IfdTypeTrait<IFD::SRational>::BE(const uint8_t* b, size_t) noexcept
+inline IFD::ORSRational IfdTypeTrait<IFD::ORSRational>::BE(const uint8_t* b, size_t) noexcept
 {
-    IFD::SRational r;
+    IFD::ORSRational r;
     r.num = BE32(b);
     r.denom = BE32(b + 4);
     return r;
 }
 
+/** @brief An entry in the IfdDir. */
 class IfdEntry
 {
 public:
-	/** Ref (ie shared pointer) */
+	/** @brief IfdEntry reference (ie shared pointer) */
 	typedef std::shared_ptr<IfdEntry> Ref;
 
-	IfdEntry(uint16_t _id, int16_t _type, int32_t _count,
-			 uint32_t _data,
-			 IfdFileContainer &_container);
+	IfdEntry(uint16_t _id, int16_t _type, int32_t _count, uint32_t _data,
+					 const IfdDir& _dir, bool synthetic = false);
 	virtual ~IfdEntry();
 
+	/** @brief id (i.e. tag) of the entry */
+	int16_t id() const noexcept
+		{
+			return m_id;
+		}
+	/** @brief Type of the entry data
+	 *
+	 * The value is from ExifTagType and is size like it is stored in the file.
+	 */
 	int16_t type() const noexcept
 		{
 			return m_type;
 		}
 
-	/** the count of items in the entry */
+	/** @brief The count of items in the entry. */
 	uint32_t count() const noexcept
 		{
 			return m_count;
 		}
 
-	/** the offset of the data. It can just be the value
-	 * if the entry is self contained.
+	/** @brief The offset of the data.
+	 *
+	 *  It can just be the value if the entry is self contained.
 	 */
 	off_t offset() noexcept
 		{
@@ -191,115 +237,61 @@ public:
 			return IfdTypeTrait<uint32_t>::BE((uint8_t*)&m_data, sizeof(uint32_t));
 		}
 
+	/** @brief Get the endian from the owning IfdDir */
 	RawContainer::EndianType endian() const;
 
-public:
-  MetaValue* make_meta_value();
-	/**
-	 * Unit size for type
-	 */
-	static size_t type_unit_size(IFD::ExifTagType _type);
-	/** load the data for the entry
-	 * if all the data fits in m_data, it is a noop
-	 * @param unit_size the size of 1 unit of data
-	 * @return true if success.
-	 */
-	bool loadData(size_t unit_size);
-
-
-	/** get the array values of type T
-	 * @param T the type of the value needed
-	 * @param array the storage
-	 * @throw whatever is thrown
-	 */
-	template <typename T>
-	Option<std::vector<T>> getArray()
+	/** @brief Return the raw data pointer */
+	const uint8_t* dataptr() const
 		{
-			try {
-				std::vector<T> array;
-				array.reserve(m_count);
-				for (uint32_t i = 0; i < m_count; i++) {
-					array.push_back(IfdTypeTrait<T>::get(*this, i));
-				}
-				return Option<decltype(array)>(array);
+			if (m_dataptr == nullptr) {
+				return (uint8_t*)&m_data;
+			} else {
+				return m_dataptr;
 			}
-			catch(const std::exception & e)
-			{
-				LOGERR("Exception: %s\n", e.what());
-			}
-			return Option<std::vector<T>>();
 		}
-	uint32_t getIntegerArrayItem(int idx);
 
+	/**
+	 * @brief Unit size for type
+	 */
+	static size_t typeUnitSize(IFD::ExifTagType _type);
+	/** @brief Load the data for the entry if m_loaded is false
+	 *
+	 * If all the data fits in m_data, it will keep it there
+	 * it will set m_loaded in case of success.
+	 * @param unit_size the size of 1 unit of data
+	 * @param offset offset from the begining of the container. Usually 0.
+	 * @return true m_loaded
+	 */
+	bool loadData(size_t unit_size, off_t offset);
+
+	/** @brief Load data into a buffer.
+	 *
+	 * @param dataptr The target byte buffer.
+	 * @param data_size the size of the buffer.
+	 * @param offset An extra offset to load the data from. For a standard IFD it is 0.
+	 * @return Return the size effectively loaded.
+	 */
+	size_t loadDataInto(uint8_t* dataptr, size_t data_size, off_t offset) const;
+	/** @brief Set the data of the entry */
+	void setData(const uint8_t* dataptr, size_t data_size);
 private:
 	uint16_t m_id;
 	uint16_t m_type;
 	uint32_t m_count;
-	uint32_t m_data; /**< raw data without endian conversion */
+	uint32_t m_data; /**< @brief raw data without endian conversion */
 	bool m_loaded;
 	uint8_t *m_dataptr;
-	IfdFileContainer & m_container;
+	const IfdDir& m_dir;
 	template <typename T> friend struct IfdTypeTrait;
 
-	/** private copy constructor to make sure it is not called */
-	IfdEntry(const IfdEntry& f);
-	/** private = operator to make sure it is never called */
-	IfdEntry & operator=(const IfdEntry&);
+	IfdEntry(const IfdEntry& f) = delete;
+	IfdEntry & operator=(const IfdEntry&) = delete;
 
 };
 
-
-
-/** get the value of type T
- * @param T the type of the value needed
- * @param idx the index, by default 0
- * @param ignore_type if true, don't check type. *DANGEROUS* Default is false.
- * @return the value
- * @throw BadTypeException in case of wrong typing.
- * @throw OutOfRangeException in case of subscript out of range
- */
-template <typename T>
-T IfdTypeTrait<T>::get(IfdEntry & e, uint32_t idx, bool ignore_type)
-	noexcept(false)
-{
-	/* format undefined means that we don't check the type */
-	if(!ignore_type && (e.m_type != IFD::EXIF_FORMAT_UNDEFINED)) {
-		if (e.m_type != IfdTypeTrait<T>::type) {
-			throw BadTypeException();
-		}
-	}
-	if (idx + 1 > e.m_count) {
-		throw OutOfRangeException();
-	}
-	if (!e.m_loaded) {
-		e.m_loaded = e.loadData(IfdTypeTrait<T>::size);
-		if (!e.m_loaded) {
-			throw TooBigException();
-		}
-	}
-	uint8_t *data;
-	if (e.m_dataptr == NULL) {
-		data = (uint8_t*)&e.m_data;
-	}
-	else {
-		data = e.m_dataptr;
-	}
-	data += (IfdTypeTrait<T>::size * idx);
-	T val;
-	if (e.endian() == RawContainer::ENDIAN_LITTLE) {
-		val = IfdTypeTrait<T>::EL(data, e.m_count - idx);
-	}
-	else {
-		val = IfdTypeTrait<T>::BE(data, e.m_count - idx);
-	}
-	return val;
-}
-
-
+/** @} */
 }
 }
-
-
 /*
   Local Variables:
   mode:c++
@@ -311,6 +303,3 @@ T IfdTypeTrait<T>::get(IfdEntry & e, uint32_t idx, bool ignore_type)
   fill-column:80
   End:
 */
-#endif
-
-
