@@ -1,7 +1,8 @@
+/* -*- mode:c++; indent-tabs-mode:nil; c-basic-offset:4; tab-width:4; -*- */
 /*
  * libopenraw - testsuite.cpp
  *
- * Copyright (C) 2008-2016 Hubert Figuiere
+ * Copyright (C) 2008-2020 Hubert Figuière
  * Copyright (C) 2008 Novell, Inc.
  *
  * This library is free software: you can redistribute it and/or
@@ -54,6 +55,7 @@
 #define IN_TESTSUITE
 #include <libopenraw/debug.h>
 #include <libopenraw/consts.h>
+#include <libopenraw/ifd.h>
 #include <libopenraw/metadata.h>
 
 #include "rawfile.hpp"
@@ -61,7 +63,7 @@
 #include "thumbnail.hpp"
 #include "bitmapdata.hpp"
 #include "metavalue.hpp"
-#include "cfapattern.hpp"
+#include "mosaicinfo.hpp"
 
 #include "xmlhandler.h"
 #include "testsuite.h"
@@ -81,6 +83,10 @@ using OpenRaw::Thumbnail;
 
 using std::unique_ptr;
 
+ThumbnailDeleter td;
+RawDataDeleter rd;
+RawFileDeleter rfd;
+
 #define RETURN_TEST_EQUALS(a,b) \
     {                               \
         bool _success = (a == b);   \
@@ -136,10 +142,10 @@ using std::unique_ptr;
         return _success;                 \
     }
 
-#define RETURN_FAIL(message,expected)    \
+#define RETURN_FAIL(message)    \
     {                                    \
-        fprintf(stderr, "FAILED: %s with '%s', expected '%s'\n", \
-                __FUNCTION__, message, expected.c_str()); \
+        fprintf(stderr, "FAILED: %s with '%s'\n", \
+                __FUNCTION__, message); \
         return false;                    \
     }
 
@@ -165,8 +171,8 @@ std::string cfaPatternToString(::or_cfa_pattern t)
     case OR_CFA_PATTERN_GRBG:
         return "GRBG";
 
-    default:
-        break;
+    case _OR_CFA_PATTERN_INVALID:
+        return "INVALID";
     }
     return "";
 }
@@ -176,9 +182,18 @@ bool equalDataType(const std::string & result, BitmapData::DataType t)
 {
     bool equal = false;
     switch(t) {
+    case OR_DATA_TYPE_NONE:
+        equal = (result == "NONE");
+        break;
+    case OR_DATA_TYPE_UNKNOWN:
+        equal = (result == "UNKNOWN");
+        break;
     case OR_DATA_TYPE_PIXMAP_8RGB:
         equal = (result == "8RGB");
         break;
+    case OR_DATA_TYPE_PIXMAP_16RGB:
+        equal = (result == "16RGB");
+        break;
     case OR_DATA_TYPE_JPEG:
         equal = (result == "JPEG");
         break;
@@ -194,8 +209,6 @@ bool equalDataType(const std::string & result, BitmapData::DataType t)
     case OR_DATA_TYPE_COMPRESSED_RAW:
         equal = (result == "COMP_RAW");
         break;
-    default:
-        break;
     }
     return equal;
 }
@@ -203,7 +216,8 @@ bool equalDataType(const std::string & result, BitmapData::DataType t)
 }
 
 Test::Test()
-    : m_total(0)
+    : m_download_disabled(false)
+    , m_total(0)
     , m_success(0)
     , m_failure(0)
 {
@@ -215,38 +229,45 @@ Test::~Test()
 
 bool Test::testRawType(const std::string & result)
 {
-    RawFile::Type t = m_rawfile->type();
+    or_rawfile_type t = or_rawfile_get_type(m_rawfile.get());
 
     // test the detection by content....
-    RawFile::Type t2;
-    OpenRaw::IO::File f(m_file.c_str());
-    ::or_error err = f.open();
-    if(err != OR_ERROR_NONE) {
-        RETURN_FAIL("failed to open",
-                    boost::lexical_cast<std::string>(err));
+    or_rawfile_type t2;
+    FILE* f = fopen(m_file.c_str(), "rb");
+    if (!f) {
+        std::string message("failed to open ");
+        message += boost::lexical_cast<std::string>(errno);
+        RETURN_FAIL(message.c_str());
     }
-    off_t len = f.filesize();
-    unique_ptr<uint8_t[]> buff(new uint8_t[len]);
-    int res = f.read(buff.get(), len);
-    if(res == len) {
-        unique_ptr<RawFile> r2(RawFile::newRawFileFromMemory(buff.get(), len));
-        if(!r2) {
-            RETURN_FAIL("failed to load from memory", std::string());
+    fseek(f, 0, SEEK_END);
+    off_t len = ftell(f);
+    fseek(f, 0, SEEK_SET);
+    auto buff = std::make_unique<uint8_t[]>(len);
+    int res = fread(buff.get(), 1, len, f);
+    fclose(f);
+    if (res == len) {
+        unique_ptr<_RawFile, RawFileDeleter> r2(or_rawfile_new_from_memory(buff.get(), len, OR_RAWFILE_TYPE_UNKNOWN), rfd);
+        if (!r2) {
+            RETURN_FAIL("failed to load from memory");
         }
-        t2 = r2->type();
-        if(t2 != t) {
-            RETURN_FAIL("type mismatch", result);
+        t2 = or_rawfile_get_type(r2.get());
+        if (t2 != t) {
+            RETURN_FAIL("type mismatch");
         }
     }
     else {
-        RETURN_FAIL("failed to load into buffer",
-                    boost::lexical_cast<std::string>(res));
+        std::string message("failed to load into buffer ");
+        message += boost::lexical_cast<std::string>(res);
+        RETURN_FAIL(message.c_str());
     }
 
     switch(t) {
     case OR_RAWFILE_TYPE_CR2:
         RETURN_TEST(result == "CR2", result);
         break;
+    case OR_RAWFILE_TYPE_CR3:
+        RETURN_TEST(result == "CR3", result);
+        break;
     case OR_RAWFILE_TYPE_CRW:
         RETURN_TEST(result == "CRW", result);
         break;
@@ -280,7 +301,17 @@ bool Test::testRawType(const std::string & result)
     case OR_RAWFILE_TYPE_RAF:
         RETURN_TEST(result == "RAF", result);
         break;
-    default:
+    case OR_RAWFILE_TYPE_GPR:
+        RETURN_TEST(result == "GPR", result);
+        break;
+    case OR_RAWFILE_TYPE_SR2:
+        RETURN_TEST(result == "SR2", result);
+        break;
+    case OR_RAWFILE_TYPE_TIFF:
+        RETURN_TEST(result == "TIFF", result);
+        break;
+    case OR_RAWFILE_TYPE_UNKNOWN:
+        RETURN_TEST(result == "UNKNOWN", result);
         break;
     }
     RETURN_TEST(false, result);
@@ -288,31 +319,40 @@ bool Test::testRawType(const std::string & result)
 
 bool Test::testRawTypeId(const std::string & result)
 {
-    RETURN_TEST_EQUALS_N(m_rawfile->typeId(),
-                         boost::lexical_cast<uint32_t>(result));
+    auto type_id = or_rawfile_get_typeid(m_rawfile.get());
+    auto vendor_id = or_rawfile_get_vendorid(m_rawfile.get());
+    bool test = false;
+    CHECK_TEST_EQUALS_N(OR_GET_FILE_TYPEID_VENDOR(type_id), vendor_id, test);
+    if (!test) {
+        return test;
+    }
+    CHECK_TEST_EQUALS_N(type_id, boost::lexical_cast<uint32_t>(result), test);
+
+    return test;
 }
 
 
 bool Test::testThumbNum(const std::string & result)
 {
-    const std::vector<uint32_t> & thumbs = m_rawfile->listThumbnailSizes();
-    int num = thumbs.size();
+    size_t num = 0;
+    /*auto thumbs =*/ or_rawfile_get_thumbnail_sizes(m_rawfile.get(), &num);
     try {
-        RETURN_TEST_EQUALS_N(num, boost::lexical_cast<int>(result));
+        RETURN_TEST_EQUALS_N(num, boost::lexical_cast<size_t>(result));
     }
     catch(...)
     {
     }
-    RETURN_FAIL("conversion failed", result);
+    RETURN_FAIL("conversion failed");
 }
 
-bool Test::testThumbSizes(const std::string & result)
+bool Test::testThumbSizes(const std::string& result)
 {
-    std::vector<uint32_t> thumbs = m_rawfile->listThumbnailSizes();
-    std::vector< std::string > v;
+    size_t num = 0;
+    auto thumbs = or_rawfile_get_thumbnail_sizes(m_rawfile.get(), &num);
+    std::vector<std::string> v;
     boost::split(v, result, boost::is_any_of(" "));
-    if(v.size() != thumbs.size()) {
-        RETURN_FAIL("mismatch number of elements", result);
+    if (v.size() != num) {
+        RETURN_FAIL("mismatch number of elements");
     }
     std::vector<uint32_t> v2;
     for (const auto & s : v)
@@ -322,26 +362,35 @@ bool Test::testThumbSizes(const std::string & result)
         }
         catch(...)
         {
-            RETURN_FAIL("conversion failed", result);
+            RETURN_FAIL("conversion failed");
         }
     }
-    RETURN_TEST(std::equal(thumbs.cbegin(), thumbs.cend(), v2.cbegin()), result);
+    bool success = true;
+    for (size_t i = 0; i < num; i++) {
+        bool test = false;
+        CHECK_TEST_EQUALS_N(thumbs[i], v2[i], test);
+        if (!test) {
+            success = false;
+        }
+    }
+    return success;
 }
 
 bool Test::testThumbFormats(const std::string & result)
 {
     bool success = true;
-    auto thumbs = m_rawfile->listThumbnailSizes();
+    size_t num = 0;
+    auto thumbs = or_rawfile_get_thumbnail_sizes(m_rawfile.get(), &num);
     std::vector< std::string > v;
     boost::split(v, result, boost::is_any_of(" "));
     auto result_iter = v.cbegin();
-    if(v.size() != thumbs.size()) {
-        RETURN_FAIL("mismatch number of elements", result);
+    if (v.size() != num) {
+        RETURN_FAIL("mismatch number of elements");
     }
-    for (const auto& thumb : thumbs) {
-        Thumbnail t;
-        m_rawfile->getThumbnail(thumb, t);
-        success &= equalDataType(*result_iter, t.dataType());
+    for (size_t i = 0; i < num; i++) {
+        unique_ptr<_Thumbnail, ThumbnailDeleter> t(or_thumbnail_new(), td);
+        or_rawfile_get_thumbnail(m_rawfile.get(), thumbs[i], t.get());
+        success &= equalDataType(*result_iter, or_thumbnail_format(t.get()));
         result_iter++;
     }
     RETURN_TEST(success, result);
@@ -350,35 +399,38 @@ bool Test::testThumbFormats(const std::string & result)
 bool Test::testThumbDataSizes(const std::string & result)
 {
     bool success = true;
-    auto thumbs = m_rawfile->listThumbnailSizes();
+    size_t num = 0;
+    auto thumbs = or_rawfile_get_thumbnail_sizes(m_rawfile.get(), &num);
     std::vector< std::string > v;
     boost::split(v, result, boost::is_any_of(" "));
     auto result_iter = v.cbegin();
-    if(v.size() != thumbs.size()) {
-        RETURN_FAIL("mismatch number of elements", result);
+    if (v.size() != num) {
+        RETURN_FAIL("mismatch number of elements");
     }
-    for (const auto& thumb : thumbs) {
-        Thumbnail t;
-        m_rawfile->getThumbnail(thumb, t);
+    for (size_t i = 0; i < num; i++) {
+        unique_ptr<_Thumbnail, ThumbnailDeleter> t(or_thumbnail_new(), td);
+        or_rawfile_get_thumbnail(m_rawfile.get(), thumbs[i], t.get());
         try {
-            success &= (boost::lexical_cast<uint32_t>(*result_iter) == t.size());
+            bool succ = false;
+            CHECK_TEST_EQUALS_N(or_thumbnail_data_size(t.get()), boost::lexical_cast<uint32_t>(*result_iter), succ);
+            success &= succ;
             result_iter++;
         }
         catch(...) {
-            RETURN_FAIL("conversion failed", result);
+            RETURN_FAIL("conversion failed");
         }
     }
     RETURN_TEST(success, result);
 }
 
 namespace {
-uint32_t computeCrc(const Thumbnail * thumb)
+uint32_t computeCrc(ORThumbnailRef thumb)
 {
     boost::crc_optimal<16, 0x1021, 0xFFFF, 0, false, false>  crc_ccitt2;
 
-    const uint8_t * data = static_cast<uint8_t *>(thumb->data());
-    size_t data_len = thumb->size();
-    crc_ccitt2 = std::for_each( data, data + data_len, crc_ccitt2 );
+    const uint8_t * data = static_cast<uint8_t *>(or_thumbnail_data(thumb));
+    size_t data_len = or_thumbnail_data_size(thumb);
+    crc_ccitt2 = std::for_each(data, data + data_len, crc_ccitt2);
     return crc_ccitt2();
 }
 }
@@ -386,48 +438,49 @@ uint32_t computeCrc(const Thumbnail * thumb)
 bool Test::testThumbMd5(const std::string & result)
 {
     bool success = true;
-    auto thumbs = m_rawfile->listThumbnailSizes();
+    size_t num = 0;
+    auto thumbs = or_rawfile_get_thumbnail_sizes(m_rawfile.get(), &num);
     std::vector< std::string > v;
     boost::split(v, result, boost::is_any_of(" "));
     auto result_iter = v.cbegin();
-    if(v.size() != thumbs.size()) {
-        RETURN_FAIL("mismatch number of elements", result);
+    if (v.size() != num) {
+        RETURN_FAIL("mismatch number of elements");
     }
-    for (const auto& thumb : thumbs) {
-        Thumbnail t;
-        m_rawfile->getThumbnail(thumb, t);
+    for (size_t i = 0; i < num; i++) {
+        unique_ptr<_Thumbnail, ThumbnailDeleter> t(or_thumbnail_new(), td);
+        or_rawfile_get_thumbnail(m_rawfile.get(), thumbs[i], t.get());
         try {
             bool succ = false;
-            uint32_t crc = computeCrc(&t);
+            uint32_t crc = computeCrc(t.get());
             CHECK_TEST_EQUALS(boost::lexical_cast<std::string>(crc), (*result_iter), succ);
             success &= succ;
             result_iter++;
         }
         catch(...) {
-            RETURN_FAIL("conversion failed", result);
+            RETURN_FAIL("conversion failed");
         }
     }
     RETURN_TEST(success, result);
 }
 
 namespace {
-unique_ptr<RawData> loadRawData(const unique_ptr<RawFile> & file)
+unique_ptr<_RawData, RawDataDeleter> loadRawData(const unique_ptr<_RawFile, RawFileDeleter> & file)
 {
-    unique_ptr<RawData> rawdata(new RawData());
+    unique_ptr<_RawData, RawDataDeleter> rawdata(or_rawdata_new(), rd);
     ::or_error err;
-    err = file->getRawData(*rawdata, OR_OPTIONS_NONE);
-    if(OR_ERROR_NONE != err) {
+    err = or_rawfile_get_rawdata(file.get(), rawdata.get(), OR_OPTIONS_NONE);
+    if (OR_ERROR_NONE != err) {
         rawdata.reset();
     }
     return rawdata;
 }
 
-uint32_t computeCrc(const unique_ptr<RawData> & rawdata)
+uint32_t computeCrc(_RawData* rawdata)
 {
     boost::crc_optimal<16, 0x1021, 0xFFFF, 0, false, false>  crc_ccitt2;
 
-    auto data = static_cast<const uint8_t *>(rawdata->data());
-    size_t data_len = rawdata->size();
+    auto data = static_cast<const uint8_t *>(or_rawdata_data(rawdata));
+    size_t data_len = or_rawdata_data_size(rawdata);
     crc_ccitt2 = std::for_each( data, data + data_len, crc_ccitt2 );
     return crc_ccitt2();
 }
@@ -436,44 +489,44 @@ uint32_t computeCrc(const unique_ptr<RawData> & rawdata)
 
 bool Test::testRawDataType(const std::string & result)
 {
-    if(m_rawdata == NULL) {
+    if (!m_rawdata) {
         m_rawdata = loadRawData(m_rawfile);
-        if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+        if (!m_rawdata) {
+            RETURN_FAIL("failed to get rawData");
         }
     }
-    RETURN_TEST(equalDataType(result, m_rawdata->dataType()), result);
+    RETURN_TEST(equalDataType(result, or_rawdata_format(m_rawdata.get())), result);
 }
 
 
 bool Test::testRawDataSize(const std::string & result)
 {
-    if(m_rawdata == NULL) {
+    if (!m_rawdata) {
         m_rawdata = loadRawData(m_rawfile);
-        if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+        if (!m_rawdata) {
+            RETURN_FAIL("failed to get rawData");
         }
     }
     try {
-        RETURN_TEST_EQUALS_N(m_rawdata->size(), boost::lexical_cast<uint32_t>(result));
+        RETURN_TEST_EQUALS_N(or_rawdata_data_size(m_rawdata.get()), boost::lexical_cast<uint32_t>(result));
     }
     catch(...) {
     }
-    RETURN_FAIL("conversion failed", result);
+    RETURN_FAIL("conversion failed");
 }
 
 bool Test::testRawDataDimensions(const std::string & result)
 {
-    if(m_rawdata == NULL) {
+    if (!m_rawdata) {
         m_rawdata = loadRawData(m_rawfile);
-        if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+        if (!m_rawdata) {
+            RETURN_FAIL("failed to get rawData");
         }
     }
     std::vector< std::string > v;
     boost::split(v, result, boost::is_any_of(" "));
     if(v.size() != 2) {
-        RETURN_FAIL("mismatch number of elements from expected result", result);
+        RETURN_FAIL("mismatch number of elements from expected result");
     }
     uint32_t x, y;
     bool success = true;
@@ -481,30 +534,33 @@ bool Test::testRawDataDimensions(const std::string & result)
         x = boost::lexical_cast<uint32_t>(v[0]);
         y = boost::lexical_cast<uint32_t>(v[1]);
         bool succ = false;
-        CHECK_TEST_EQUALS_N(m_rawdata->width(), x, succ);
+        uint32_t rx, ry;
+        rx = ry = 0;
+        or_rawdata_dimensions(m_rawdata.get(), &rx, &ry);
+        CHECK_TEST_EQUALS_N(rx, x, succ);
         success &= succ;
-        CHECK_TEST_EQUALS_N(m_rawdata->height(), y,  succ);
+        CHECK_TEST_EQUALS_N(ry, y,  succ);
         success &= succ;
     }
     catch(...)
     {
-        RETURN_FAIL("conversion failed", result);
+        RETURN_FAIL("conversion failed");
     }
     RETURN_TEST(success, result)
 }
 
-bool Test::testRawDataRoi(const std::string & result)
+bool Test::testRawDataActiveArea(const std::string & result)
 {
     if(m_rawdata == NULL) {
         m_rawdata = loadRawData(m_rawfile);
         if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+            RETURN_FAIL("failed to get rawData");
         }
     }
     std::vector< std::string > v;
     boost::split(v, result, boost::is_any_of(" "));
     if(v.size() != 4) {
-        RETURN_FAIL("mismatch number of elements from expected result", result);
+        RETURN_FAIL("mismatch number of elements from expected result");
     }
     uint32_t x, y, w, h;
     try {
@@ -515,11 +571,12 @@ bool Test::testRawDataRoi(const std::string & result)
     }
     catch(...)
     {
-        RETURN_FAIL("conversion failed", result);
+        RETURN_FAIL("conversion failed");
     }
-    RETURN_TEST(x == m_rawdata->roi_x() && y == m_rawdata->roi_y()
-        && w == m_rawdata->roi_width() && h == m_rawdata->roi_height(),
-        result)
+    uint32_t rx, ry, rw, rh;
+    rx = ry = rw = rh = 0;
+    or_rawdata_get_active_area(m_rawdata.get(), &rx, &ry, &rw, &rh);
+    RETURN_TEST(x == rx && y == ry && w == rw && h == rh, result);
 }
 
 bool Test::testRawCfaPattern(const std::string & result)
@@ -527,12 +584,11 @@ bool Test::testRawCfaPattern(const std::string & result)
     if(m_rawdata == NULL) {
         m_rawdata = loadRawData(m_rawfile);
         if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+            RETURN_FAIL("failed to get rawData");
         }
     }
     bool succ = false;
-    CHECK_TEST_EQUALS(cfaPatternToString(
-                          m_rawdata->cfaPattern()->patternType()),
+    CHECK_TEST_EQUALS(cfaPatternToString(or_rawdata_get_cfa_pattern_type(m_rawdata.get())),
                       result, succ);
     return succ;
 }
@@ -542,7 +598,7 @@ bool Test::testRawMinValue(const std::string & result)
     if(m_rawdata == NULL) {
         m_rawdata = loadRawData(m_rawfile);
         if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+            RETURN_FAIL("failed to get rawData");
         }
     }
     uint16_t expected;
@@ -551,9 +607,12 @@ bool Test::testRawMinValue(const std::string & result)
     }
     catch(...)
     {
-        RETURN_FAIL("conversion failed", result);
+        RETURN_FAIL("conversion failed");
     }
-    RETURN_TEST_EQUALS_N(m_rawdata->blackLevel(), expected);
+    uint16_t black, white;
+    black = white = 0;
+    or_rawdata_get_levels(m_rawdata.get(), &black, &white);
+    RETURN_TEST_EQUALS_N(black, expected);
 }
 
 
@@ -562,7 +621,7 @@ bool Test::testRawMaxValue(const std::string & result)
     if(m_rawdata == NULL) {
         m_rawdata = loadRawData(m_rawfile);
         if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+            RETURN_FAIL("failed to get rawData");
         }
     }
     uint16_t expected;
@@ -571,9 +630,12 @@ bool Test::testRawMaxValue(const std::string & result)
     }
     catch(...)
     {
-        RETURN_FAIL("conversion failed", result);
+        RETURN_FAIL("conversion failed");
     }
-    RETURN_TEST_EQUALS_N(m_rawdata->whiteLevel(), expected);
+    uint16_t black, white;
+    black = white = 0;
+    or_rawdata_get_levels(m_rawdata.get(), &black, &white);
+    RETURN_TEST_EQUALS_N(white, expected);
 }
 
 bool Test::testRawMd5(const std::string & result)
@@ -581,11 +643,11 @@ bool Test::testRawMd5(const std::string & result)
     if(m_rawdata == NULL) {
         m_rawdata = loadRawData(m_rawfile);
         if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+            RETURN_FAIL("failed to get rawData");
         }
     }
 
-    uint32_t crc = computeCrc(m_rawdata);
+    uint32_t crc = computeCrc(m_rawdata.get());
 
     uint32_t expected = 0;
     try {
@@ -593,18 +655,18 @@ bool Test::testRawMd5(const std::string & result)
     }
     catch(...)
     {
-        RETURN_FAIL("conversion failed", result);
+        RETURN_FAIL("conversion failed");
     }
     RETURN_TEST_EQUALS_N(crc, expected);
 }
 
 
-bool Test::testRawDecompressedMd5(const std::string & result)
+bool Test::testRawDecompressedMd5(const std::string &)
 {
     if(m_rawdata == NULL) {
         m_rawdata = loadRawData(m_rawfile);
         if(m_rawdata == NULL) {
-            RETURN_FAIL("failed to get rawData", result);
+            RETURN_FAIL("failed to get rawData");
         }
     }
     return false;
@@ -613,63 +675,51 @@ bool Test::testRawDecompressedMd5(const std::string & result)
 
 bool Test::testMetaOrientation(const std::string & result)
 {
-    int32_t orientation = m_rawfile->getOrientation();
+    int32_t orientation = or_rawfile_get_orientation(m_rawfile.get());
     RETURN_TEST_EQUALS_N(orientation, boost::lexical_cast<int32_t>(result));
 }
 
 
 bool Test::testExifString(int32_t meta_index, const std::string & result)
 {
-    auto val = m_rawfile->getMetaValue(meta_index);
+    auto val = or_rawfile_get_metavalue(m_rawfile.get(), meta_index);
     if (val) {
         //
-        auto stringVal = val->getString(0);
+        auto stringVal = std::string(or_metavalue_get_string(val, 0));
         RETURN_TEST_EQUALS(stringVal, result);
     }
-    RETURN_FAIL("meta data not found", result);
+    RETURN_FAIL("meta data not found");
 }
 
 bool Test::testMakerNoteCount(const std::string & result)
 {
     try {
-        OpenRaw::Internals::IfdFile & ifd_file =
-            dynamic_cast<OpenRaw::Internals::IfdFile &>(*m_rawfile.get());
-        auto exif = ifd_file.exifIfd();
-        auto maker_note = exif->getMakerNoteIfd();
-        if (!maker_note) {
-            RETURN_FAIL("no maker not found", result);
-        }
-        RETURN_TEST_EQUALS_N(maker_note->numTags(),
+        auto ifd = or_rawfile_get_ifd(m_rawfile.get(), OR_IFD_MNOTE);
+        auto numTags = or_ifd_count_tags(ifd);
+        or_ifd_release(ifd);
+        RETURN_TEST_EQUALS_N(numTags,
                              boost::lexical_cast<int32_t>(result));
     }
     catch(const std::bad_cast & e) {
-        RETURN_FAIL("not an IFD file", result);
+        RETURN_FAIL("not an IFD file");
     }
     catch(...) {
-        RETURN_FAIL("unknown exception", result);
+        RETURN_FAIL("unknown exception");
     }
 }
 
 bool Test::testMakerNoteId(const std::string & result)
 {
-    try {
-        OpenRaw::Internals::IfdFile& ifd_file =
-            dynamic_cast<OpenRaw::Internals::IfdFile&>(*m_rawfile.get());
-        auto exif = ifd_file.exifIfd();
-        auto maker_note =
-            std::dynamic_pointer_cast<OpenRaw::Internals::MakerNoteDir>(
-                exif->getMakerNoteIfd());
-        if (!maker_note) {
-            RETURN_FAIL("no maker not found", result);
-        }
-        RETURN_TEST_EQUALS(maker_note->getId(), result);
+    auto ifd = or_rawfile_get_ifd(m_rawfile.get(), OR_IFD_MNOTE);
+    if (!ifd) {
+        RETURN_FAIL("no MakerNote found");
     }
-    catch(const std::bad_cast & e) {
-        RETURN_FAIL("not an IFD file", result);
-    }
-    catch(...) {
-        RETURN_FAIL("unknown exception", result);
+    auto makernote_id = or_ifd_get_makernote_id(ifd);
+    or_ifd_release(ifd);
+    if (!makernote_id) {
+        RETURN_FAIL("no MakeNote id");
     }
+    RETURN_TEST_EQUALS(std::string(makernote_id), result);
 }
 
 
@@ -687,10 +737,10 @@ int Test::run()
         fprintf(stderr, "File not found, skipping. (%d)\n", errno);
         return 0;
     }
-    m_rawfile.reset(RawFile::newRawFile(m_file.c_str()));
+    m_rawfile.reset(or_rawfile_new(m_file.c_str(), OR_RAWFILE_TYPE_UNKNOWN));
 
     if(m_rawfile == NULL) {
-        RETURN_FAIL("m_rawfile == NULL", std::string("not NULL"));
+        RETURN_FAIL("m_rawfile == NULL");
     }
 
     for(const auto & elem : m_results) {
@@ -727,8 +777,8 @@ int Test::run()
         case XML_rawDataDimensions:
             pass = testRawDataDimensions(elem.second);
             break;
-        case XML_rawDataRoi:
-            pass = testRawDataRoi(elem.second);
+        case XML_rawDataActiveArea:
+            pass = testRawDataActiveArea(elem.second);
             break;
         case XML_rawCfaPattern:
             pass = testRawCfaPattern(elem.second);
@@ -774,6 +824,18 @@ int Test::run()
             m_success++;
         }
     }
+    if (!m_to_run.empty()) {
+        fprintf(stderr, "Not all test have been run. Still to run: ");
+        for (int t : m_to_run) {
+            auto iter = testsuitetagnames.find(t);
+            if (iter != testsuitetagnames.end()) {
+                fprintf(stderr, "%s, ", iter->second.c_str());
+            } else {
+                fprintf(stderr, "%d, ", t);
+            }
+        }
+        fprintf(stderr, "\n");
+    }
     fprintf(stderr, "total %d, success %d, failure %d\n", m_total,
             m_success, m_failure);
     return m_failure;
@@ -836,6 +898,7 @@ int TestSuite::load_overrides(const std::string & overrides_file)
 
 namespace {
 
+#if HAVE_CURL
 void set_file_override(xmlNode *test, const std::string & path)
 {
     xmlNode * childrens = test->children;
@@ -851,9 +914,9 @@ void set_file_override(xmlNode *test, const std::string & path)
 }
 
 
-#if HAVE_CURL
-int download(const std::string & source, CURL* handle,
-                     const std::string & download_dir, std::string & dest)
+int download(const std::string & source, const std::string& referer,
+             CURL* handle,
+             const std::string & download_dir, std::string & dest)
 {
     dest = "";
     FILE *fp = NULL;
@@ -871,6 +934,7 @@ int download(const std::string & source, CURL* handle,
 
         if(stat(dest.c_str(), &f_stat) == -1) {
             CURLcode error;
+            struct curl_slist* hlist = NULL;
             std::cout << "Downloading " << source
                       << " to " << dest << std::endl;
 
@@ -882,15 +946,33 @@ int download(const std::string & source, CURL* handle,
                 return -1;
             }
             curl_easy_setopt(handle, CURLOPT_WRITEDATA, fp);
+            if (!referer.empty()) {
+                std::string header("Referer: ");
+                header += referer;
+                hlist = curl_slist_append(hlist, header.c_str());
+                curl_easy_setopt(handle, CURLOPT_HTTPHEADER, hlist);
+
+                std::cout << "Set HTTP header " << header << std::endl;
+            }
             curl_easy_setopt(handle, CURLOPT_URL, source.c_str());
+            curl_easy_setopt(handle, CURLOPT_FAILONERROR, 1L);
+            curl_easy_setopt(handle, CURLOPT_FOLLOWLOCATION, 1L);
             error = curl_easy_perform(handle);
+            if (hlist) {
+                curl_easy_setopt(handle, CURLOPT_HTTPHEADER, nullptr);
+                curl_slist_free_all(hlist);
+            }
+            auto written = ftell(fp);
             fclose(fp);
 
-            if(!error) {
+            if (!error && written > 0) {
                 std::cout << " DONE\n";
-            }
-            else {
-                std::cout << " CURL Error " << error << std::endl;
+            } else {
+                if (error) {
+                    std::cout << " CURL Error " << error << std::endl;
+                } else {
+                    std::cout << " Empty file\n";
+                }
                 unlink(dest.c_str());
                 dest = "";
                 return -1;
@@ -929,23 +1011,27 @@ void TestSuite::walk_tests(xmlNode * testsuite, CURL* handle,
         test = test->next;
     }
 
-    for(const auto & elem : m_tests) {
+    for (const auto & elem : m_tests) {
         std::string n = elem.first;
         std::string dest;
-        int ret = download(elem.second->source(), handle, download_dir, dest);
-
-        if(ret == 0 && !dest.empty()) {
+        if (elem.second->download_disabled()) {
+            fprintf(stderr, "Skipping source %s\n",
+                    elem.second->source().c_str());
+            continue;
+        }
+        int ret = download(elem.second->source(), elem.second->referer(),
+                           handle, download_dir, dest);
+        if (ret == 0 && !dest.empty()) {
             xmlNode * test2 = NULL;
             auto iter2 = overrides.find(n);
             if (iter2 != overrides.cend()) {
                 test2 = iter2->second;
-            }
-            else {
+            } else {
                 test2 = xmlNewNode(NULL, (const xmlChar*)"test");
                 xmlAddChild(testsuite, test2);
-                xmlNode *name_node = xmlNewTextChild(test2, NULL,
-                                                     (const xmlChar*)"name",
-                                                     (const xmlChar*)n.c_str());
+                xmlNode *name_node = xmlNewTextChild(
+                    test2, NULL, (const xmlChar*)"name",
+                    (const xmlChar*)n.c_str());
                 xmlAddChild(test2, name_node);
             }
             set_file_override(test2, dest);
@@ -1034,10 +1120,6 @@ int main(int argc, char ** argv)
 {
     bool bootstrap = false;
     std::string download_dir;
-    const char * srcdir = getenv("srcdir");
-    if(srcdir == NULL) {
-        srcdir = "./";
-    }
 
     int opt;
     while ((opt = getopt(argc, argv, "bd:")) != -1) {
@@ -1073,10 +1155,18 @@ int main(int argc, char ** argv)
         }
     }
 
-    std::string testsuite_file = srcdir;
-    testsuite_file += "/";
-    testsuite_file += (argv[optind] ? argv[optind] : "testsuite.xml");
+        const char * srcdir = getenv("srcdir");
+    if(srcdir == NULL) {
+        srcdir = "./";
+    }
 
+    std::string testsuite_file;
+    if (argv[optind]) {
+        testsuite_file = argv[optind];
+    } else {
+        testsuite_file = srcdir;
+        testsuite_file += "/testsuite.xml";
+    }
     or_debug_set_level(ERROR);
 
     TestSuite testsuite;
@@ -1097,6 +1187,8 @@ int main(int argc, char ** argv)
   mode:c++
   c-file-style:"stroustrup"
   c-file-offsets:((innamespace . 0))
+  c-basic-offset: 4
+  tab-width: 4
   indent-tabs-mode:nil
   fill-column:80
   End:
